SystemOrganization addCategory: #'AntSimulation-Test'!
SystemOrganization addCategory: #'AntSimulation-Graphics'!
SystemOrganization addCategory: #AntSimulation!

TestCase subclass: #EnvironmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/3/2012 23:37'!
testAddEntity
	|env entity cell entityStatus|
	env := Environment new initWithRows: 10 columns: 10.

	entity := Entity new.
	cell := env cellAtRow: 3 column: 2.
	env add: entity to: cell.
	
	self assert: env entities size equals: 1.

	entityStatus := env statusForEntity: entity.
	self assert: entityStatus health equals: 1.0 .
	
	self assert: (entityStatus cell) == cell.
	self assert: (entityStatus cell) row equals: 3.
	self assert: (entityStatus cell) column equals: 2.! !

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/3/2012 23:32'!
testCreation
	|env|
	env := Environment new initWithRows: 10 columns: 15.
	self assert: env rowCount equals: 10.
	self assert: env columnCount equals: 15.
	self assert: env entities size equals: 0.
! !

TestCase subclass: #SurfaceEnvironmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!SurfaceEnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/4/2012 00:27'!
testGenerateRandom
	|env|
	env := SurfaceEnvironment generateRandomWithRows: 25 columns: 25 antCount: 10.
	
	self assert: (env entities) size equals: 10.
	
	(env entities) do: [:entity |
		|entityStatus cell|
		entityStatus := env statusForEntity: entity.
		self assert: entityStatus ~= nil.
		
		cell := entityStatus cell.
		self assert: ((cell entities) includes: entity).		
		self assert: (env cellAtRow: (cell row) column: (cell column)) == cell.
	].! !

SimpleButtonMorph subclass: #AntSimulationCell
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!AntSimulationCell commentStamp: 'agallan2 7/4/2012 20:59' prior: 0!
I represent a single discrete location in the AntSimulation. I am a button, as clicking me returns information about the simulation object I represent.!

!AntSimulationCell methodsFor: 'accessing' stamp: 'agallan2 7/5/2012 08:25'!
defaultLabel
	^ ' '.! !

!AntSimulationCell methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 08:52'!
initialize
	super initialize.
	self borderWidth: 2.
	bounds := 0@0 corner: 16@16.! !

BorderedMorph subclass: #GraphicalEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!GraphicalEnvironment methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 08:26'!
initialize
	super initialize.! !

GraphicalEnvironment subclass: #SurfaceEnvironmentView
	instanceVariableNames: 'cells'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Graphics'!

!SurfaceEnvironmentView methodsFor: 'accessing' stamp: 'agallan2 7/5/2012 08:38'!
cellsPerSide
	^ 30.! !

!SurfaceEnvironmentView methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 09:00'!
initialize
	| sampleCell width height n |
	super initialize.
	n := self cellsPerSide.
	sampleCell := AntSimulationCell new.
	width := sampleCell width.
	height := sampleCell height.
	self bounds: (5@5 extent: ((width*n) @(height*n)) + (2 * self borderWidth)).
	cells := Matrix new: n tabulate: [ :i :j | self newCellAt: i at: j ].! !

!SurfaceEnvironmentView methodsFor: 'initialization' stamp: 'agallan2 7/5/2012 08:53'!
newCellAt: i at: j
	"Creates a new AntSimulationCell at the given i,j coordinates."
	| c origin |
	c := AntSimulationCell new.
	origin := self innerBounds origin.
	self addMorph: c.
	c position: ((i - 1) * c width) @ ((j - 1) * c height) + origin.
! !

Object subclass: #AntSensoryInput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Object subclass: #AntSimulation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

AntSimulation class
	instanceVariableNames: 'random'!

AntSimulation class
	instanceVariableNames: 'random'!

!AntSimulation class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/4/2012 00:09'!
random
	(random == nil) ifTrue: [random := Random seed: 1].
	^random! !

Object subclass: #Cell
	instanceVariableNames: 'row column entities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Cell methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/3/2012 23:03'!
add: entity
	entities add: entity! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:38'!
column
	^column! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/4/2012 00:25'!
entities
	^entities! !

!Cell methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:24'!
initWithRow: newRow column: newColumn
 	row := newRow.
	column := newColumn.
	entities := Set new.! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:38'!
row
	^row! !

Object subclass: #Entity
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Entity subclass: #ActiveEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

ActiveEntity subclass: #Ant
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Entity methodsFor: 'comparing' stamp: 'VanceThornton 7/3/2012 18:58'!
= anEntity
	self == anEntity ifTrue: [^true].
	^(id = anEntity id)
! !

!Entity methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:58'!
id
	^id! !

!Entity methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:13'!
initialize
	id := UUID new.! !

Entity subclass: #InactiveEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

InactiveEntity subclass: #Food
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Object subclass: #EntityStatus
	instanceVariableNames: 'cell health'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:36'!
cell
	^cell! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:35'!
health
	^health! !

!EntityStatus methodsFor: 'initialize-release' stamp: 'VanceThornton 7/3/2012 23:14'!
initWithCell: aCell
	cell := aCell.
	health := 1.0.
	! !

Object subclass: #Environment
	instanceVariableNames: 'cellMatrix entityStatusMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/3/2012 23:32'!
add: entity to: cell
	|status|
	status := EntityStatus new initWithCell: cell.
	entityStatusMap add: (Association key: entity value: status).
	cell add: entity.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 00:56'!
cellAtRow: row column: column
	^cellMatrix at: row at: column
	! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:40'!
columnCount
	^cellMatrix columnCount! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:15'!
entities
	^entityStatusMap keys! !

!Environment methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:06'!
initWithRows: rowCount columns: columnCount
	cellMatrix := Matrix rows: rowCount columns: columnCount.
	cellMatrix indicesDo: [:row :column| cellMatrix at: row at: column put: (Cell new initWithRow: row column: column)].
	entityStatusMap := Dictionary new.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:40'!
rowCount
	^cellMatrix rowCount! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:16'!
statusForEntity: entity
	^entityStatusMap at: entity! !

Environment subclass: #SurfaceEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!SurfaceEnvironment class methodsFor: 'class initialization' stamp: 'VanceThornton 7/4/2012 00:14'!
generateRandomWithRows: rows 
	columns: columns 
	antCount: antCount
	
	|env|

	env := SurfaceEnvironment new initWithRows: rows columns: columns.
	
	antCount timesRepeat: [
		|ant cell row column|
		ant := Ant new.
		row := AntSimulation random nextInt: rows.
		column := AntSimulation random nextInt: columns.
		cell := env cellAtRow: row column: column.
		env add: ant to: cell.
	].
	^env! !
